"""Things that are needed for convenient parallel Execution"""

from PyFoam.Basics.Utilities import Utilities
from PyFoam.FoamInformation import foamMPI
from os import path
from string import strip
import sys

class LAMMachine(Utilities):
    """Wrapper class for starting an stopping a LAM-Machine"""
    def __init__(self,machines):
        """@param machines: Name of the file with the machine information"""

        Utilities.__init__(self)

        self.stop()
        self.mFile=machines
        self.boot()
        if not self.machineOK():
            print "Error: LAM was not started"
            sys.exit(-1)
            
    def machineOK(self):
        """Check whether the LAM machine was properly booted"""
        if self.running:
            if(foamMPI()=="LAM"):
                if self.cpuNr()<=0:
                    self.running=False

        return self.running

    def stop(self):
        """Stops a LAM-machine (if one is running)"""
        self.running=False
        self.execute("lamhalt -v")
        
    def boot(self):
        """Boots a LAM-machine using the machine-file"""
        if(foamMPI()=="LAM"):
            self.execute("lamboot -s -v "+self.mFile)
            self.running=True
        elif(foamMPI()=="OPENMPI"):
            self.running=True
        else:
            print " Unknown or missing MPI-Implementation: ",foamMPI()
            sys.exit(-1)
            
    def cpuNr(self):
        if(foamMPI()=="LAM"):
            if self.running:
                lines=self.execute("lamnodes")
                nr=0
                for l in lines:
                    tmp=l.split(':')
                    if len(tmp)>1:
                        nr+=int(tmp[1])
                return nr
            else:
                return -1
        elif(foamMPI()=="OPENMPI"):
            f=open(self.mFile)
            l=map(strip,f.readlines())
            f.close()
            nr=0
            for m in l:
                tmp=m.split()
                if len(tmp)==1:
                    nr+=1
                elif len(tmp)==0:
                    pass
                else:
                    print "Machinefile not valid (I think): more than one element in one line:",tmp
                    sys.exit(-1)
            return nr

    def buildMPIrun(self,argv):
        """Builds a list with a working mpirun command (for that MPI-Implementation)
        @param argv: the original arguments that are to be wrapped
        @return: list with the correct mpirun-command"""
        
        nr=str(self.cpuNr())
        mpirun=["mpirun","-np",nr]

        if(foamMPI()=="LAM"):
            pass
        elif(foamMPI()=="OPENMPI"):
            mpirun+=["--mca","pls_rsh_agent","rsh","-machinefile",self.mFile,"-x","LD_LIBRARY_PATH","-x","WM_PROJECT_DIR","-x","FOAM_MPI_LIBBIN","-x","MPI_BUFFER_SIZE"]
        else:
            print " Unknown or missing MPI-Implementation for mpirun: ",foamMPI()
            sys.exit(-1)
        mpirun+=argv[0:3]+["-parallel"]+argv[3:]

        return mpirun
    
    def writeMetis(self,sDir):
        """Write the parameter-File for a metis decomposition
        @param sDir: Solution directory
        @type sDir: PyFoam.RunDictionary.SolutionDirectory"""

        params="method metis;\n"

        self.writeDecomposition(sDir,params)

    def writeSimple(self,sDir,direction):
        """Write the parameter-File for a metis decomposition
        @param sDir: Solution directory
        @type sDir: PyFoam.RunDictionary.SolutionDirectory
        @param direction: direction in which to decompose (0=x, 1=y, 2=z)"""

        params ="method simple;\n"
        params+="\nsimpleCoeffs\n{\n\t n \t ("
        if direction==0:
            params+=str(self.cpuNr())+" "
        else:
            params+="1 "
        if direction==1:
            params+=str(self.cpuNr())+" "
        else:
            params+="1 "
        if direction==2:
            params+=str(self.cpuNr())
        else:
            params+="1"
        params+=");\n\t delta \t 0.001;\n}\n"
        
        self.writeDecomposition(sDir,params)

    def writeDecomposition(self,sDir,par):
        """Write parameter file for a decomposition
        @param par:Parameters specific for that kind of decomposition
        @type par:str
        @param sDir: Solution directory
        @type sDir: PyFoam.RunDictionary.SolutionDirectory"""

        f=open(path.join(sDir.systemDir(),"decomposeParDict"),"w")
        self.writeDictionaryHeader(f)
        f.write("// * * * * * * * * * //\n\n")
        f.write("numberOfSubdomains "+str(self.cpuNr())+";\n\n")
        f.write(par)
        f.write("\n\n// * * * * * * * * * //")
        f.close()
